<?php

declare(strict_types=1);

namespace Drupal\Tests\ui_patterns\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\ui_patterns\Plugin\UiPatterns\PropType\AttributesPropType;

/**
 * @coversDefaultClass \Drupal\ui_patterns\Plugin\UiPatterns\PropType\LinksPropType
 *
 * @group ui_patterns
 */
final class AttributesPropTypeNormalizationTest extends UnitTestCase {

  /**
   * @covers ::normalize
   *
   * @dataProvider provideNormalizationData
   */
  public function testNormalize(array $value, array $expected): void {
    $normalized = AttributesPropType::normalize($value);
    self::assertEquals($normalized, $expected);
  }

  /**
   * Provide data for testNormalize.
   */
  public static function provideNormalizationData(): \Generator {
    $data = [
      "Empty value" => [
        "value" => [],
        "expected" => [],
      ],
      "Standardized primitives, so already OK" => self::standardizedPrimitives(),
      "Type transformations" => self::typeTransformation(),
      "List array" => self::listArray(),

    ];
    foreach ($data as $label => $test) {
      yield $label => [
        $test['value'],
        $test['expected'],
      ];
    };
  }

  /**
   * Standardized primitives, so already OK.
   */
  protected static function standardizedPrimitives() {
    $value = [
      "foo" => "bar",
      "string" => "Lorem ipsum",
      "array" => [
        "One",
        "Two",
        3,
      ],
      "integer" => 4,
      "float" => 1.4,
    ];
    $expected = $value;
    return [
      "value" => $value,
      "expected" => $expected,
    ];
  }

  /**
   * Type transformations.
   */
  protected static function typeTransformation() {
    $value = [
      "true_boolean" => TRUE,
      "false_boolean" => FALSE,
      "null_boolean" => NULL,
      "markup" => "Hello <b>World</b>",
      "associative_array" => [
        "Un" => "One",
        "Deux" => "Two",
        "Trois" => 3,
      ],
      "nested_array" => [
        "One",
        [
          "Two",
          "Three",
        ],
        [
          "deep" => [
            "very deep" => ["foo", "bar"],
          ],
        ],
      ],
    ];
    $expected = [
      "true_boolean" => "1",
      "false_boolean" => "",
      "null_boolean" => "",
      "markup" => "Hello World",
      "associative_array" => [
        "One",
        "Two",
        3,
      ],
      "nested_array" => [
        "One",
        // JSON encoding because we don't know how deep is the nesting.
        '["Two","Three"]',
        '{"deep":{"very deep":["foo","bar"]}}',
      ],
    ];
    return [
      "value" => $value,
      "expected" => $expected,
    ];
  }

  /**
   * List array.
   */
  protected static function listArray() {
    $value = [
      "One",
      "Two",
      3,
    ];
    // This doesn't look like a valid HTML attribute structure, but we rely on
    // Drupal Attribute object normalization here.
    $expected = [
      "0" => "One",
      "1" => "Two",
      "2" => 3,
    ];
    return [
      "value" => $value,
      "expected" => $expected,
    ];
  }

}
